@@ -22,6 +22,15 @@ MODEL_DIR_NAMES = {'IR': 'Nominal_rates',
 FILE_MARK = "file::"
 
 
+# Custom Exception class for sensi validation
+class SensiSyntaxError(Exception):
+    def __init__(self, msg):
+        self.msg = str(msg)
+
+    def __str__(self):
+        return self.msg
+
+
 class Syntax:
     def __init__(self, expression, col, condition, value):
         self.expression = expression
@@ -32,7 +41,7 @@ class Syntax:
 
 def extract_value_from_equal(param_string):
     if "=" not in param_string:
-        raise SyntaxError("Incorrect syntax in param. Unable to find equal.")
+        raise SensiSyntaxError("Incorrect syntax in param. Unable to find equal.")
 
     last_equal_position = param_string.rindex("=")
     syntax = param_string[:last_equal_position].strip('"')
@@ -49,7 +58,7 @@ def extract_target_column(param_string):
         value = param_string[left_quote_position+1:right_quote_position]
         return syntax, value
     else:
-        raise SyntaxError("Incorrect syntax in param. Unable to find square quote at the end of syntax.")
+        raise SensiSyntaxError("Incorrect syntax in param. Unable to find square quote at the end of syntax.")
 
 
 def parse_param(input_syntax):
@@ -66,29 +75,23 @@ def parse_param(input_syntax):
 
     param_string, param_value = extract_value_from_equal(input_syntax)
 
-    if param_string.startswith(FILE_MARK):
-        param_string = param_string[len(FILE_MARK):]
-        # Checks if '.where' exists in param_string
-        if ".where" in param_string:
-            param_expression, param_condition = param_string.split(".where")
-            # param_condition = param_condition.strip("()")
-        else:
-            # # print("Didn't find '.where' clause in {}".format(param_string))
-            param_expression = param_string
-
-        # Gets the column in the para_expressions
-        param_expression, param_col = extract_target_column(param_expression)
-        param_names = re.findall(r'\[.+?\]', param_expression)
-
-        result = "$"
-        for name in param_names:
-            result = "".join([result, "..*[@.name is '{}']".format(name.strip('[]'))])
-
-        result = "".join([result, re.split(r'\[.+?\]', param_expression)[-1].strip('[]'), '.filename'])
-        param_expression = result
+    param_string = param_string[len(FILE_MARK):]
+    # Checks if '.where' exists in param_string
+    if ".where" in param_string:
+        param_expression, param_condition = param_string.split(".where")
     else:
-        pass
-        # # print("file:: not found in param: {}, skipping".format(param_string))
+        param_expression = param_string
+
+    # Gets the column in the para_expressions
+    param_expression, param_col = extract_target_column(param_expression)
+    param_names = re.findall(r'\[.+?\]', param_expression)
+
+    result = "$"
+    for name in param_names:
+        result = "".join([result, "..*[@.name is '{}']".format(name.strip('[]'))])
+
+    result = "".join([result, re.split(r'\[.+?\]', param_expression)[-1].strip('[]'), '.filename'])
+    param_expression = result
 
     syntax = Syntax(param_expression, param_col, param_condition, param_value)
     return syntax
@@ -97,50 +100,60 @@ def parse_param(input_syntax):
 def query(data, expression):
     result = []
     if data and expression:
-        try:
-            tree = Tree(data)
-            result = list(tree.execute(expression))
+        if expression.startswith("$"):
+            try:
+                tree = Tree(data)
+                result = tree.execute(expression)
+                # If result is not a string (i.e. generator)
+                # convert results to a list
+                # In general: generators are returned when using an expression that contains ..* operator
+                #             itertools can also be returned with just using .. operator
+                if not isinstance(result, str):
+                    result = list(result)
 
-        except AttributeError as err:
-            # print("In Tree(data): ", err.message)
-            pass
-        except StopIteration:
-            # print("In tree.execute(expression): StopIteration")
-            pass
-        except SyntaxError:
-            # print("In tree.execute(expression): SyntaxError")
-            pass
+            except AttributeError as err:
+                raise SensiSyntaxError("In Tree(data): ", err.message)
+
+            except StopIteration:
+                raise SensiSyntaxError("In tree.execute(expression): StopIteration")
+
+            except SyntaxError:
+                raise SensiSyntaxError("In tree.execute(expression): SyntaxError")
 
     elif data is None:
-        # print("Data is None")
-        pass
+        raise SensiSyntaxError("Empty Data in query function")
+
     else:
-        # # print("Expression is None")
-        pass
+        raise SensiSyntaxError("Empty Expression in query function")
 
     return result
 
 
 def get_input_file_path(data, expression, env_dir):
-    input_file_path = None
     filename = query(data, expression)
-    folder_names = re.findall(r"(?<=')\w+(?=')", expression)
-
-    for index in range(len(folder_names)):
-        if folder_names[index] in MODEL_DIR_NAMES:
-            folder_names[index] = MODEL_DIR_NAMES[folder_names[index]]
+    eco_name, driver_name = re.findall(r"(?<=')\w+(?=')", expression)
+    driver_name = MODEL_DIR_NAMES[driver_name]
 
     if filename:
-        local_filepath = "/".join(folder_names + filename)
-        file_path = os.path.join(env_dir, 'resources/Central/RN_inputs', local_filepath).replace('\\', '/')
+        eco_folder_id = query(data, "$.framework..*[@.name is '{}']".format(eco_name))
+        if eco_folder_id:
+            local_filepath = "/".join([eco_folder_id[0]['folder_id'], driver_name, filename[0]])
+            # fetch root and input_root names from settings.json
+            folder_id = query(data, "$.framework.sensi_1.folder_id")
+            input_root_name = "RN_inputs" if query(data, "$.framework.name") == "RN" else "RW_inputs"
+            file_path = os.path.join(env_dir, 'resources', folder_id, input_root_name, local_filepath).replace('\\', '/')
+            if not os.path.exists(file_path):
+                folder_name = query(data, "$.framework.sensi_1.name")
+                file_path = os.path.join(env_dir, 'resources', folder_name, input_root_name, local_filepath).replace('\\', '/')
+                if not os.path.exists(file_path):
+                    raise SensiSyntaxError("Unable to find input file {}".format(file_path))
+
+            return file_path
 
-        if not os.path.exists(file_path):
-            # print("The input file can't be found at {}".format(file_path))
-            pass
         else:
-            input_file_path = file_path
-
-    return input_file_path
+            raise SensiSyntaxError("Unable to find {} folder_id in settings.json".format(eco_name))
+    else:
+        raise SensiSyntaxError(f"Unable to find input file from expression {expression} in {env_dir}")
 
 
 def get_selection_from_dataframe(selection, dataframe):
@@ -151,7 +164,6 @@ def get_selection_from_dataframe(selection, dataframe):
         elif col.count(',') == 0:
             column, row = col, None
         else:
-            # print("Couldn't find a correct cols")
             column, row = None, None
 
         if column:
@@ -163,20 +175,20 @@ def get_selection_from_dataframe(selection, dataframe):
                     if column == "*":
                         pass
                     elif column.isnumeric():
-                        dataframe = dataframe[[dataframe.columns[int(column)]]]
+                        if int(column) >= 1:
+                            dataframe = dataframe[[dataframe.columns[int(column)-1]]]
+                        else:
+                            raise SensiSyntaxError("Unable to select because the {} column is less than 1".format(column))
                     else:
-                        # TODO par Quincy: Throw error when incorrect column/row value
-                        # print("{} is not correct".format(column))
-                        pass
+                        raise SensiSyntaxError("Unable to select because the {} column is not an int or *".format(column))
                 else:
-                    # print("{} is not correct".format(column))
-                    pass
+                    raise SensiSyntaxError("Unable to select because the {} column is not defined correctly".format(column))
+
             except KeyError:
-                # # print("{} is not in {} columns".format(column, dataframe))
-                pass
+                raise SensiSyntaxError("Unable to select because the {} column is not a valid column".format(column))
+
             except IndexError:
-                # # print("index {} is out of bounds for {} columns".format(column, dataframe))
-                pass
+                raise SensiSyntaxError("Unable to select because the {} column is out of bounds".format(column))
 
         if row:
             try:
@@ -187,19 +199,21 @@ def get_selection_from_dataframe(selection, dataframe):
                     if row == "*":
                         pass
                     elif row.isnumeric():
-                        dataframe = dataframe.iloc[[int(row)], :]
+                        if int(row) >= 1:
+                            dataframe = dataframe.iloc[[int(row)-1], :]
+                        else:
+                            raise SensiSyntaxError("Unable to select because the {} row is less than 1".format(row))
                     else:
-                        # print("{} is not correct".format(row))
-                        pass
+                        # TODO par Quincy: Throw error when incorrect column/row value
+                        raise SensiSyntaxError("Unable to select because the {} row is not an int or *".format(row))
                 else:
-                    # print("{} is not correct".format(row))
-                    pass
+                    raise SensiSyntaxError("Unable to select because the {} row is not defined correctly".format(row))
+
             except KeyError:
-                # # print("{} is not in {} rows".format(row, dataframe))
-                pass
+                raise SensiSyntaxError("Unable to select because {} is not a valid row".format(row))
+
             except IndexError:
-                # # print("index {} is out of bounds for {} rows".format(row, dataframe))
-                pass
+                raise SensiSyntaxError("Unable to select because the row {} is out of bounds".format(row))
 
     return dataframe
 
@@ -226,35 +240,28 @@ def select_from_dataframe(condition, operation, dataframe):
                                 else:
                                     pass
 
-                try:
-                    if operation == "==":
-                        dataframe = dataframe[selected.T.iloc[0].isin(values)]
-                    elif operation == "!=":
-                        dataframe = dataframe[~selected.T.iloc[0].isin(values)]
-                    elif operation == ">=":
-                        dataframe = dataframe[selected.T.iloc[0] >= values[0]]
-                    elif operation == ">":
-                        dataframe = dataframe[selected.T.iloc[0] > values[0]]
-                    elif operation == "<=":
-                        dataframe = dataframe[selected.T.iloc[0] <= values[0]]
-                    elif operation == "<":
-                        dataframe = dataframe[selected.T.iloc[0] < values[0]]
-                    else:
-                        # print("Unsupported Operation!")
-                        pass
-                except Exception:
-                    dataframe = pd.DataFrame()
-                    pass
+                if operation == "==":
+                    dataframe = dataframe[selected.T.iloc[0].isin(values)]
+                elif operation == "!=":
+                    dataframe = dataframe[~selected.T.iloc[0].isin(values)]
+                elif operation == ">=":
+                    dataframe = dataframe[selected.T.iloc[0] >= values[0]]
+                elif operation == ">":
+                    dataframe = dataframe[selected.T.iloc[0] > values[0]]
+                elif operation == "<=":
+                    dataframe = dataframe[selected.T.iloc[0] <= values[0]]
+                elif operation == "<":
+                    dataframe = dataframe[selected.T.iloc[0] < values[0]]
+                else:
+                    raise SensiSyntaxError(" {} is an unsupported Operation!".format(operation))
 
-                # print(dataframe)
                 return dataframe
 
             else:
-                # print('No rvalue found in {}'.format(condition))
-                pass
+                raise SensiSyntaxError('No rvalue found in {}'.format(condition))
+
     else:
-        # print('No lvalue found in {}'.format(condition))
-        pass
+        raise SensiSyntaxError('No lvalue found in {}'.format(condition))
 
 
 def interpret_condition(condition, dataframe):
@@ -273,8 +280,7 @@ def interpret_condition(condition, dataframe):
         elif condition.count('<') == 1:
             dataframe = select_from_dataframe(condition, "<", dataframe)
         else:
-            # print("{} is not a correct condition".format(condition))
-            pass
+            raise SensiSyntaxError("{} is not a correct condition".format(condition))
 
     return dataframe
 
@@ -318,8 +324,8 @@ def apply_value_to_selection(value, selected_dict):
                 for column in selected_dict.keys():
                     selected_dict[column] = {k: v / value if not isinstance(value, (str)) and not isinstance(v, str) else 0 for k, v in selected_dict[column].items()}
             else:
-                # print("ERROR")
-                pass
+                raise SensiSyntaxError(" {} is an unsupported Operation!".format(operation))
+
         else:
             for column in selected_dict.keys():
                 selected_dict[column] = {k: value for k, v in selected_dict[column].items()}
@@ -364,10 +370,15 @@ def apply_syntax_to_file(input_path, syntax, settings_json):
 
                 if syntax.col:
                     # print(syntax.col)
-                    if syntax.condition:
-                        selected_df = get_selection_from_dataframe(syntax.col, df_concat)
-                    else:
-                        selected_df = get_selection_from_dataframe(syntax.col, input_df)
+                    try:
+                        if syntax.condition:
+                            selected_df = get_selection_from_dataframe(syntax.col, df_concat)
+                        else:
+                            selected_df = get_selection_from_dataframe(syntax.col, input_df)
+                    except SensiSyntaxError as err:
+                        index = err.msg.find("because")
+                        final_msg = err.msg[:index] + 'from ' + input_path + ' ' + err.msg[index:]
+                        raise SensiSyntaxError(final_msg)
 
                     # print("Column: ", syntax.col, end=" ")
 
@@ -387,24 +398,21 @@ def apply_syntax_to_file(input_path, syntax, settings_json):
                     # print(input_df)
                     os.remove(input_path)
                     input_df.to_csv(input_path, sep=col_sep, index=False)
-
                     return True
-
                 else:
-                    # print("No col found for selection")
-                    pass
+                    raise SensiSyntaxError("No col found for selection")
             else:
-                # print("Couldn't find the input file at ", input_path)
-                pass
+                raise SensiSyntaxError("Unable to find the input file {}".format(input_path))
         else:
-            # print("Couldn't find the seps from settings.json")
-            pass
+            raise SensiSyntaxError("Unable to find the input_format in settings.json")
     else:
-        # print("Error in apply_syntax_to_file arguments:")
-        # if not input_path: print("input_path is None")
-        # if not syntax: print("syntax is None")
-        # if not settings_json: print("settings_json is None")
-        pass
+        msg = []
+        if input_path is None:
+            raise SensiSyntaxError("Unable to find input file to apply syntax")
+        if syntax is None:
+            raise SensiSyntaxError("No syntax to apply to file")
+        if settings_json is None:
+            raise SensiSyntaxError("Unable to read settings.json")
 
     return False
 
